' **********************************************************************
' Solution to a nonlinear equation by the Newton-Raphson method.
'
' Example:
'
' F(X) = X * Ln(X) - 1 = 0
'
' Derivative: F'(X) = Ln(X) + 1
'
' True Solution is X = 1.763222834...
' **********************************************************************

#INCLUDE "fbmath.bi"

' **********************************************************************
' Define number of iterations and precision
' **********************************************************************

CONST MaxIter = 1000  ' Max number of iterations
CONST Tol     = 1D-6  ' Required precision

' **********************************************************************
' Define starting point
' **********************************************************************

DIM AS DOUBLE X = 2

' **********************************************************************
' Define the function
' **********************************************************************

FUNCTION Func(X AS DOUBLE) AS DOUBLE
  Func = X * LOG(X) - 1
END FUNCTION

' **********************************************************************
' Define the derivative
' **********************************************************************

FUNCTION Deriv(X AS DOUBLE) AS DOUBLE
  Deriv = LOG(X) + 1
END FUNCTION

' **********************************************************************
' Main program
' **********************************************************************

DIM AS DOUBLE F  ' Function

NewtEq @Func, @Deriv, X, MaxIter, Tol, F

SELECT CASE MathErr()
  CASE OptNonConv
    PRINT "Non-convergence!"
    STOP
  CASE OptSing
    PRINT "Null derivative!"
    STOP
END SELECT

CLS

PRINT "Solution to nonlinear equation (Newton-Raphson method)"
PRINT "------------------------------------------------------"
PRINT

PRINT "Root:": PRINT

PRINT USING "X = ####.######"; X

PRINT : PRINT "Function value:" : PRINT

PRINT USING "F = ####.######"; F

SLEEP

