' **********************************************************************
' Solution to a nonlinear equation by the secant method.
'
' Example:
'
' F(X) = X * Ln(X) - 1 = 0

' True Solution is X = 1.763222834...
' **********************************************************************

#INCLUDE "fbmath.bi"

' **********************************************************************
' Define number of iterations and precision
' **********************************************************************

CONST MaxIter = 1000  ' Max number of iterations
CONST Tol     = 1D-6  ' Required precision

' **********************************************************************
' Define starting points
' **********************************************************************

DIM AS DOUBLE X = 1, Y = 2

' **********************************************************************
' Define the function
' **********************************************************************

FUNCTION Func(X AS DOUBLE) AS DOUBLE
  Func = X * LOG(X) - 1
END FUNCTION

' **********************************************************************
' Main program
' **********************************************************************

DIM AS DOUBLE F  ' Function

Secant @Func, X, Y, MaxIter, Tol, F

IF MathErr() = OptNonConv THEN
    PRINT "Non-convergence!"
    STOP
END IF

CLS

PRINT "Solution to nonlinear equation (Secant method)"
PRINT "----------------------------------------------"
PRINT

PRINT "Root:": PRINT

PRINT USING "X = ####.######"; X

PRINT : PRINT "Function value:" : PRINT

PRINT USING "F = ####.######"; F

SLEEP
