' ******************************************************************
' Test of Beta function
' ******************************************************************

#INCLUDE "fbmath.bi"

FUNCTION NDigits (Approx AS DOUBLE, Exact AS DOUBLE) AS INTEGER
' Returns the number of correct digits found in Approx

  DIM AS DOUBLE R

  IF Approx = Exact THEN
    NDigits = 16                     ' For double precision
    EXIT FUNCTION
  END IF

  R = ABS((Approx - Exact) / Exact)  ' Relative deviation
  NDigits = INT(0.5 - Log10(R))
END FUNCTION

' ------------------------------------------------
' Reference data for Beta function
' (modified from 'Numerical Recipes Example Book')
' ------------------------------------------------

CONST N = 17  ' Number of values

DIM AS DOUBLE X(1 TO N) = _
{ 1.0#, 0.2#, 1.0#, 0.4#, 1.0#, 0.6#, 0.8#, 6.0#, _
  6.0#, 6.0#, 6.0#, 6.0#, 7.0#, 5.0#, 4.0#, 3.0#, 2.0#}

DIM AS DOUBLE Y(1 TO N) = _
{ 1.0#, 1.0#, 0.2#, 1.0#, 0.4#, 1.0#, 1.0#, 6.0#, _
  5.0#, 4.0#, 3.0#, 2.0#, 7.0#, 5.0#, 4.0#, 3.0#, 2.0#}

DIM AS DOUBLE Ref(1 TO N) =  _  ' Beta(X,Y) reference values
{ 1#, 5#, 5#, 2.5#, 2.5#,    _
  1.6666666666666666667#,    _
  1.25#,                     _
  0.36075036075036075036D-3, _
  0.79365079365079365079D-3, _
  0.19841269841269841270D-2, _
  0.59523809523809523810D-2, _
  0.23809523809523809524D-1, _
  0.83250083250083250083D-4, _
  0.15873015873015873016D-2, _
  0.71428571428571428571D-2, _
  0.33333333333333333333D-1, _
  0.16666666666666666667#}

DIM AS INTEGER I
DIM AS DOUBLE  R, Z

PRINT "   X      Y        Beta(X, Y)           Reference            Digits"
PRINT "-------------------------------------------------------------------"

FOR I = 1 TO N
  Z = Beta(X(I), Y(I))
  PRINT USING "###.#  "; X(I); Y(I);
  PRINT USING "####.###############"; Z; Ref(I);
  PRINT SPACE$(10); NDigits(Z, Ref(I))
NEXT I

SLEEP
