' ******************************************************************
' Test of DiGamma and TriGamma functions
' ******************************************************************

#INCLUDE "fbmath.bi"

' ------------------------------------------------------------------
' Reference data (computed with Maple)
' ------------------------------------------------------------------

CONST N = 10  ' Number of values

DIM AS DOUBLE X(1 TO N) = _
{ 0.01#, 0.1#, 0.5#, 1#, 1.5#, 2#, 3#, 5#, 10#, 100#}

DIM AS DOUBLE DG(1 TO N) =       _  ' DiGamma(X) reference values
{ -100.56088545786867450#,       _
   -10.423754940411076795#,      _
    -1.9635100260214234794#,     _
     -0.57721566490153286061#,   _
      0.36489973978576520559D-1, _
      0.42278433509846713939#,   _
      0.92278433509846713939#,   _
      1.5061176684318004727#,    _
      2.2517525890667211076#,    _
      4.6001618527380874002#}

DIM AS DOUBLE TG(1 TO N) =       _  ' TriGamma(X) reference values
{10001.621213528313220#,         _
   101.43329915079275882#,       _
     4.9348022005446793094#,     _
     1.6449340668482264365#,     _
     0.93480220054467930942#,    _
     0.64493406684822643647#,    _
     0.39493406684822643647#,    _
     0.22132295573711532536#,    _
     0.10516633568168574612#,    _
     0.10050166663333571395D-1}

FUNCTION NDigits (Approx AS DOUBLE, Exact AS DOUBLE) AS INTEGER
' Returns the number of correct digits found in Approx

  DIM AS DOUBLE R

  IF Approx = Exact THEN
    NDigits = 16                     ' For double precision
    EXIT FUNCTION
  END IF

  R = ABS((Approx - Exact) / Exact)  ' Relative deviation
  NDigits = INT(0.5 - Log10(R))
END FUNCTION

DIM AS INTEGER I
DIM AS DOUBLE  R, Z

CLS

PRINT "   X          DiGamma(X)               Reference            Digits"
PRINT "------------------------------------------------------------------"

FOR I = 1 TO N
  Z = DiGamma(X(I))

  PRINT USING "###.##   "; X(I);
  PRINT USING "######.###############"; Z; DG(I);
  PRINT SPACE$(10); NDigits(Z, DG(I))
NEXT I

LOCATE 25,1: PRINT "Press a key...";

SLEEP

CLS

PRINT "   X          TriGamma(X)              Reference            Digits"
PRINT "------------------------------------------------------------------"

FOR I = 1 TO N
  Z = TriGamma(X(I))

  PRINT USING "###.##   "; X(I);
  PRINT USING "######.###############"; Z; TG(I);
  PRINT SPACE$(10); NDigits(Z, TG(I))
NEXT I

SLEEP
