' ******************************************************************
' Test of Error function
' ******************************************************************

#INCLUDE "fbmath.bi"

FUNCTION NDigits (Approx AS DOUBLE, Exact AS DOUBLE) AS INTEGER
' Returns the number of correct digits found in Approx

  DIM AS DOUBLE R

  IF Approx = Exact THEN
    NDigits = 16                     ' For double precision
    EXIT FUNCTION
  END IF

  R = ABS((Approx - Exact) / Exact)  ' Relative deviation
  NDigits = INT(0.5 - Log10(R))
END FUNCTION

' ------------------------------------------------
' Reference data for error function
' (modified from 'Numerical Recipes Example Book')
' ------------------------------------------------

CONST N = 20  ' Number of values

DIM AS DOUBLE X(1 TO N) = _
{ 0.1#, 0.2#, 0.3#, 0.4#, 0.5#, 0.6#, 0.7#, 0.8#, 0.9#, 1.0#, _
  1.1#, 1.2#, 1.3#, 1.4#, 1.5#, 1.6#, 1.7#, 1.8#, 1.9#, 2.0#}

DIM AS DOUBLE Ref(1 TO N) =  _  ' Erf(X) reference values
{ 0.11246291601828489220#, _
  0.22270258921047845414#, _
  0.32862675945912742764#, _
  0.42839235504666845510#, _
  0.52049987781304653768#, _
  0.60385609084792592256#, _
  0.67780119383741847298#, _
  0.74210096470766048617#, _
  0.79690821242283212852#, _
  0.84270079294971486934#, _
  0.88020506957408169977#, _
  0.91031397822963538024#, _
  0.93400794494065243660#, _
  0.95228511976264881052#, _
  0.96610514647531072707#, _
  0.97634838334464400777#, _
  0.98379045859077456363#, _
  0.98909050163573071418#, _
  0.99279042923525746995#, _
  0.99532226501895273416#}

DIM AS INTEGER I
DIM AS DOUBLE  R, Z

PRINT "   X          Erf(X)                 Reference          Digits"
PRINT "--------------------------------------------------------------"

FOR I = 1 TO N
  Z = Erf(X(I))
  PRINT USING "###.#    "; X(I);
  PRINT USING "####.###############"; Z; Ref(I);
  PRINT SPACE$(10); NDigits(Z, Ref(I))
NEXT I

SLEEP
