' ******************************************************************
' Test of Gamma function
' ******************************************************************

#INCLUDE "fbmath.bi"

FUNCTION NDigits (Approx AS DOUBLE, Exact AS DOUBLE) AS INTEGER
' Returns the number of correct digits found in Approx

  DIM AS DOUBLE R

  IF Approx = Exact THEN
    NDigits = 16                     ' For double precision
    EXIT FUNCTION
  END IF

  R = ABS((Approx - Exact) / Exact)  ' Relative deviation
  NDigits = INT(0.5 - Log10(R))
END FUNCTION

' ------------------------------------------------
' Reference data for Gamma function
' (modified from 'Numerical Recipes Example Book')
' ------------------------------------------------

CONST N = 20  ' Number of values

DIM AS DOUBLE X(1 TO N) = _
{ 1.0#,  1.2#,  1.4#,  1.6#,  1.8#,  2.0#,  0.2#,  0.4#,  0.6#,  0.8#, _
 -0.2#, -0.4#, -0.6#, -0.8#, 10.0#, 15.0#, 20.0#, 25.0#, 30.0#, 34.0#}

DIM AS DOUBLE Ref(1 TO N) =  _
{ 1#,                        _
  0.91816874239976061064#,   _
  0.88726381750307528922#,   _
  0.89351534928769026144#,   _
  0.93138377098024269891#,   _
  1#,                        _
  4.5908437119988030532#,    _
  2.2181595437576882231#,    _
  1.4891922488128171024#,    _
  1.1642297137253033736#,    _
 -5.8211485686265168682#,    _
 -3.7229806220320427560#,    _
 -3.6969325729294803718#,    _
 -5.7385546399985038165#,    _
  362880#,                   _
  87178291200#,              _
  121645100408832000#,       _
  6.2044840173323943936D23,  _
  8.841761993739701954543616D30, _
  8.68331761881188649551819440128D36}

DIM AS INTEGER I
DIM AS DOUBLE  R, Z

PRINT "   X            Gamma(X)                  Reference              Digits"
PRINT "-----------------------------------------------------------------------"

FOR I = 1 TO N
  Z = Gamma(X(I))                          ' To test Gamma
' Z = SgnGamma(X(I)) * Exp(LnGamma(X(I)))  ' To test LnGamma

  PRINT USING "###.##   "; X(I);
  PRINT USING "     #.###############^^^^"; Z; Ref(I);
  PRINT SPACE$(5); NDigits(Z, Ref(I))
NEXT I

SLEEP
