' ******************************************************************
' Test of Incomplete Beta function
' ******************************************************************

#INCLUDE "fbmath.bi"

FUNCTION NDigits (Approx AS DOUBLE, Exact AS DOUBLE) AS INTEGER
' Returns the number of correct digits found in Approx

  DIM AS DOUBLE R

  IF Approx = Exact THEN
    NDigits = 16                     ' For double precision
    EXIT FUNCTION
  END IF

  R = ABS((Approx - Exact) / Exact)  ' Relative deviation
  NDigits = INT(0.5 - Log10(R))
END FUNCTION

' ------------------------------------------------
' Reference data for Incomplete Beta function
' (modified from 'Numerical Recipes Example Book')
' ------------------------------------------------

CONST N = 20  ' Number of values

DIM AS DOUBLE A(1 TO N) = _
{ 0.5#,  0.5#,  0.5#,  1.0#,  1.0#,  1.0#,  1.0#,  5.0#, 10.0#, 10.0#, _
 10.0#, 10.0#, 20.0#, 20.0#, 20.0#, 20.0#, 20.0#, 30.0#, 30.0#, 30.0#}

DIM AS DOUBLE B(1 TO N) = _
{ 0.5#,  0.5#, 0.5#,  0.5#,  0.5#,  0.5#,  1.0#,  5.0#,  0.5#,  5.0#, _
  5.0#, 10.0#, 5.0#, 10.0#, 10.0#, 20.0#, 20.0#, 10.0#, 10.0#, 20.0#}

DIM AS DOUBLE X(1 TO N) = _
{ 0.01#, 0.10#, 1.00#, 0.01#, 0.10#, 1.00#, 0.5#, 0.5#, 0.9#, 0.5#, _
  1.0#,  0.5#,  0.8#,  0.6#,  0.8#,  0.5#,  0.6#, 0.7#, 0.8#, 0.7#}

DIM AS DOUBLE Ref(1 TO N) =  _  ' IBeta(A, B, X) reference values
{ 0.63768560858519847915D-1, _
  0.20483276469913345164#,   _
  1#,                        _
  0.50125628933800452655D-2, _
  0.51316701949486200400D-1, _
  1#,                        _
  0.5#,                      _
  0.5#,                      _
  0.15164090963470992063#,   _
  0.89782714843750000000D-1, _
  1#,                        _
  0.5#,                      _
  0.45987732975757912179#,   _
  0.21468161023717389333#,   _
  0.95073648269578749938#,   _
  0.5#,                      _
  0.89794136871059179403#,   _
  0.22412974918083661378#,   _
  0.75864054871920859442#,   _
  0.93001290669949261326#}

DIM AS INTEGER I
DIM AS DOUBLE  R, Z

PRINT "   A      B      X       IBeta(A, B, X)         Reference            Digits"
PRINT "---------------------------------------------------------------------------"

FOR I = 1 TO N
  Z = IBeta(A(I), B(I), X(I))
  PRINT USING "###.## "; A(I); B(I); X(I);
  PRINT USING "####.###############"; Z; Ref(I);
  PRINT SPACE$(10); NDigits(Z, Ref(I))
NEXT I

SLEEP

