' ******************************************************************
' Solution of a system of linear equations by the LU method
' ******************************************************************
' The LU algorithm is applied to a linear system A * X = B, such
' that A(n x n) is the system matrix, B(n) is the constant vector
' and X(n) is the solution vector.
'
' The matrix A is factored into a product L * U, where L(n x n) is
' lower triangular and U(n x n) is upper triangular. Then the system
' L * U * X = B is solved.
'
' Function MathErr can be called to check the error code:
'   * MatOk     = no error
'   * MatErrDim = non-compatible dimensions (A is not square)
'   * MatSing   = quasi-singular matrix
' ******************************************************************

#INCLUDE "fbmath.bi"

SUB PrintMatrix(Title AS STRING, A() AS DOUBLE)

  DIM AS INTEGER I, J, Lb, Ub

  PRINT : PRINT Title : PRINT

  Lb = LBOUND(A, 1)
  Ub = UBOUND(A, 1)

  FOR I = Lb TO Ub
    FOR J = Lb TO Ub
      PRINT USING "#####.######"; A(I,J);
    NEXT J
    PRINT
  NEXT I
END SUB

SUB PrintVector(Title AS STRING, B() AS DOUBLE)

  DIM AS INTEGER I, Lb, Ub

  PRINT : PRINT Title : PRINT

  Lb = LBOUND(B)
  Ub = UBOUND(B)

  FOR I = Lb TO Ub
    PRINT USING "#####.######"; B(I)
  NEXT I
END SUB

CONST N = 4                                ' Size of matrix

DIM AS DOUBLE A(1 TO N, 1 TO N) = _        ' System matrix
{{ 2,  1,   5, -8}, _
 { 7,  6,   2,  2}, _
 {-1, -3, -10,  4}, _
 { 2,  2,   2,  1}}

DIM AS DOUBLE B(1 TO N) = {0, 17, -10, 7}  ' Constant vector

DIM AS DOUBLE X(1 TO N)                    ' Solution vector

CLS

PrintMatrix "System matrix:", A()
PrintVector "Constant vector:", B()

LU_Decomp A()

SELECT CASE MathErr()
  CASE MatOk
    LU_Solve A(), B(), X()
    PrintVector "Solution vector:", X()
  CASE MatErrDim
    PRINT "Non-compatible dimensions!"
  CASE MatSing
    PRINT "Quasi-singular matrix!"
END SELECT

SLEEP
