' ******************************************************************
' Minimization of a function of one variable by the Golden Search
' method.
'
' Example function: Func1 = EXP(-2 * X) - EXP(-X)
' The minimum is at (Ln(2), -1/4)
' ******************************************************************

#INCLUDE "fbmath.bi"

' ******************************************************************
' Define starting points, number of iterations, and precision
' ******************************************************************

CONST A = 0, B = 1    ' Starting points
CONST MaxIter = 1000  ' Max number of iterations
CONST Tol = 1D-6      ' Required precision (must be > Sqrt(MachEp))

' ******************************************************************
' Define the function to be minimized
' ******************************************************************

FUNCTION Func(X AS DOUBLE) AS DOUBLE
  Func = EXP(-2 * X) - EXP(-X)
END FUNCTION

' ******************************************************************
' Main program
' ******************************************************************

DIM AS DOUBLE TrueMinX, TrueMinY  ' True minimum
DIM AS DOUBLE Xmin, Ymin          ' Found minimum

' Compute true minimum
TrueMinX = LOG(2)
TrueMinY = Func(TrueMinX)

' Perform search
GoldSearch @Func, A, B, MaxIter, Tol, Xmin, Ymin

IF MathErr = OptNonConv THEN
  PRINT "Non-convergence!"
  STOP
END IF

' Display results
CLS
PRINT USING "Exact      :     Xmin = #####.######"; TrueMinX;
PRINT USING "     Ymin = #####.######"; TrueMinY
PRINT USING "GoldSearch :     Xmin = #####.######"; Xmin;
PRINT USING "     Ymin = #####.######"; Ymin

SLEEP

