' ******************************************************************
' Minimization of a function of several variables by simplex method.
'
' Example: Rosenbrock's function:
'
' F(X, Y) = 100 * (Y - X^2)^2 + (1 - X)^2
'
' True minimum is at (1, 1), F = 0
'
' Ref: H. Rosenbrock, Comput. J., 1960, 3, 175
' ******************************************************************

#INCLUDE "fbmath.bi"

' ******************************************************************
' Define number of variables, number of iterations, and precision
' ******************************************************************

CONST Nvar    = 2     ' Number of variables
CONST MaxIter = 1000  ' Max number of iterations
CONST Tol     = 1D-6  ' Required precision

' ******************************************************************
' Define starting point
' ******************************************************************

' Variables: X(1) = X, X(2) = Y
DIM AS DOUBLE X(1 TO Nvar) = {2, 2}

' ******************************************************************
' Define the function to be minimized
' ******************************************************************

FUNCTION Func(X() AS DOUBLE) AS DOUBLE

  DIM AS DOUBLE A, B

  A = X(2) - X(1) * X(1)
  B = 1 - X(1)

  Func = 100 * A * A + B * B
END FUNCTION

' ******************************************************************
' Main program
' ******************************************************************

DIM AS DOUBLE  Fmin  ' Function value at minimum
DIM AS INTEGER I, J  ' Loop variables

Simplex @Func, X(), MaxIter, Tol, Fmin

IF MathErr = OptNonConv THEN
  PRINT "Non-convergence!"
  STOP
END IF

CLS

PRINT "Minimization of Rosenbrock's function (simplex method)"
PRINT "------------------------------------------------------"
PRINT

PRINT "Coordinates of minimum:" : PRINT

FOR I = 1 TO Nvar
  PRINT USING "X(#) = #####.######"; I; X(I)
NEXT I

PRINT : PRINT "Function value:" : PRINT

PRINT "Fmin  = ";  Fmin

SLEEP
