' ***************************************************************************************
'Subject: Include file for large-integer arithmetic Basic library.
'Author : Sjoerd.J.Schaper
'Update : 06-06-2009
'Code   : FreeBasic 0.20.0 adaptation
'
' ***************************************************************************************
#INCLIB "largeint"
#IF __FB_LANG__= "fb"
'
' ***************************************************************************************
'                            Initialization and assignment
' ***************************************************************************************
'The LargeInt class is initialized with a 'LargeInit(size%, name$)' call at
'module level. You have to declare (constant) indices for referencing the
'large integers you're going to use (CONST p = 0, q = 1, ..), so that each
'one is associated with its own distinct pointer. These pointers are passed
'to the procedures. Don't call with same pointers ('Squ p, p'), and beware
'of duplicate references, for they will crash your program.
'
'In the following prototypes,
'letters p, q, r, d, m denote 16-bit (pseudo) pointers to largeint's;
'variables a, k, sw are 16-bit integers, |a| stands for absolute value,
'c is a 32-bit integer; g is a number string, f and h are any string.
'The t#'s within brackets denote the range of scratchpad-registers shared
'by the subroutine, that cannot be passed as arguments.
'
DECLARE FUNCTION LargeInit alias "LargeInit" (k AS SHORT, f AS ZSTRING PTR) AS SHORT
'Initialize the number-arrays, open the primetable and logfile.
'To be called with k = number of largeint's in your program and
'string f = the name of the program. If f = "" no logfile is opened.
'Return value is the maximal largeint length measured in array elements
'(='words'). The decimal number length is about 4.51 * largeint length.
'
DECLARE SUB Letf alias "Letf" (p AS SHORT, c AS INTEGER)
'Assign signed long integer c to largeint p.
'
DECLARE SUB Readst alias "Readst" (p AS SHORT, g AS ZSTRING PTR)
'Assign decimal number string g to largeint p. Except for a prefixed minus
'sign, there's no check on non-digit characters in the string. {t1..t2}
'
DECLARE SUB Rndf alias "Rndf" (p AS SHORT, k AS SHORT)
'Assign a random positive value of bitlength k to largeint p.
'
DECLARE SUB Term alias "Term" ()
'Close the primetable and logfile.
'
' ***************************************************************************************
'                                 Conversion functions
' ***************************************************************************************
DECLARE FUNCTION Decf alias "Decf" (p AS SHORT) AS INTEGER
'Convert largeint p to base MD (see below), return the bytelength of
'decimal(p), call before CnvSt. {t1..t2}
'
DECLARE SUB CnvSt alias "CnvSt" (g AS ZSTRING PTR)
'Convert base MD number to string. First call k = Decf(p),
'create stringbuffer g = STRING$(k, "0") and pass to CnvSt. {t2}
'
DECLARE FUNCTION Logf alias "Logf" (p AS SHORT) AS DOUBLE
'Return double precision natural logarithm of |largeint p|.
'
DECLARE SUB Ratdec alias "Ratdec" (g AS ZSTRING PTR, p AS SHORT, q AS SHORT)
'Convert the decimal expansion of rational p/q in [0, 1) to string. {t0..t2}
'
' ***************************************************************************************
'                                   Obtaining output
' ***************************************************************************************
#IFNDEF __VB
DECLARE SUB Printf alias "Printf" (ByRef f AS STRING, ByRef g AS STRING, _
 ByRef h AS STRING, sw AS SHORT)
'Print prefix f, number string g and suffix h to screen and the logfile,
'set switch = 1 to attach a CrLf, switch = 2 to also attach length(g).
'
DECLARE SUB Printn alias "Printn" (p AS SHORT, ByRef f AS STRING, ByRef h AS STRING, _
 sw AS SHORT)
'Print largeint p with prefix f and suffix h using the Printf switches. {t1..t2}
'
DECLARE SUB Printr alias "Printr" (p AS SHORT, q AS SHORT, ByRef f AS STRING, k AS SHORT)
'Print the decimal expansion of rational p/q with prefix f and length k. {t1..t2}
'
DECLARE SUB Prints alias "Prints" (ByRef f AS STRING, sw AS SHORT)
'Print string f. Set switch = 1 to attach a CrLf, switch = 2 for two.
'
'(Else to be replaced with VisualBasic-tailored printfunctions)
#ENDIF
DECLARE SUB Work alias "Work" (ByRef f AS STRING)
'Set string f to the working directory in ENVIRON$("LARGEINT"),
'the stringbuffer must be large enough to hold the path.
'
' ***************************************************************************************
'                              Basic arithmetic functions
' ***************************************************************************************
DECLARE SUB Subt alias "Subt" (p AS SHORT, q AS SHORT)
'Set largeint p to p - q.
'
DECLARE SUB Add alias "Add" (p AS SHORT, q AS SHORT)
'Set largeint p to p + q.
'
DECLARE SUB Inc alias "Inc" (p AS SHORT, a AS SHORT)
'Increment largeint p by signed one-word value a.
'
DECLARE SUB Divd alias "Divd" (p AS SHORT, d AS SHORT, q AS SHORT)
'Divide p by d, set p to the remainder and q to the quotient. {t2}
'
DECLARE FUNCTION Divint alias "Divint" (p AS SHORT, a AS SHORT) AS SHORT
'Divide largeint p by |one-word a|,
'return |remainder| and set p to the quotient.
'
DECLARE SUB Ldiv alias "Ldiv" (p AS SHORT, d AS SHORT)
'Set p to quotient p / d. {t1..t2}
'
DECLARE SUB Mult alias "Mult" (p AS SHORT, q AS SHORT, r AS SHORT)
'Set largeint r to p * q.
'
DECLARE SUB Lmul alias "Lmul" (p AS SHORT, q AS SHORT)
'Set p to product p * q. {t2}
'
DECLARE SUB Squ alias "Squ" (p AS SHORT, q AS SHORT)
'Set q to the square of p, faster than p * p.
'
DECLARE SUB Isqrt alias "Isqrt" (p AS SHORT, q AS SHORT)
'Set q to the truncated integer part of square root(p). {t0..t2}
'
DECLARE SUB Chs alias "Chs" (p AS SHORT)
'Change the sign of largeint p.
'
DECLARE FUNCTION Absf alias "Absf" (p AS SHORT) AS SHORT
'Clear sign bit of largeint p and return the current value.
'
' ***************************************************************************************
'                                 Copying and comparison
' ***************************************************************************************
DECLARE SUB Dup alias "Dup" (p AS SHORT, q AS SHORT)
'Copy largeint p to q.
'
DECLARE SUB Swp alias "Swp" (p AS SHORT, q AS SHORT)
'Exchange largeint's p and q.
'
DECLARE FUNCTION Cmp alias "Cmp" (p AS SHORT, q AS SHORT) AS SHORT
'Compare returns -1 if p < q, 0 if p = q, or 1 if p > q.
'
DECLARE FUNCTION Isf alias "Isf" (p AS SHORT, a AS SHORT) AS SHORT
'Boolean: check if largeint p equals one-word value a.
'
DECLARE FUNCTION Ismin1 alias "Ismin1" (p AS SHORT, m AS SHORT) AS SHORT
'Boolean: check if p = m - 1
'
' ***************************************************************************************
'                                   Bit manipulation
' ***************************************************************************************
DECLARE SUB Boolf alias "Boolf" (p AS SHORT, q AS SHORT, k AS SHORT)
'Bitwise Boolean functions set largeint p to q Op(k) p, with Op(1) = AND,
'Op(2) = XOR, Op(3) = OR. If k = 0 then p is set to NOT p, and q is ignored.
'In each case, the sign of p is unaffected.
'
DECLARE SUB Lsft alias "Lsft" (p AS SHORT, k AS SHORT)
'Left-shift largeint p by k bits; k < 0 shifts |k| full words.
'
DECLARE SUB Rsft alias "Rsft" (p AS SHORT, k AS SHORT)
'Right-shift largeint p by k bits; k < 0 shifts |k| full words.
'
DECLARE FUNCTION Odd alias "Odd" (p AS SHORT) AS SHORT
'Remove trailing zero bits from largeint p, return (right) shift count.
'
DECLARE FUNCTION Bitl alias "Bitl" (p AS SHORT) AS INTEGER
'Return the bitlength of largeint p.
'
' ***************************************************************************************
'                             Modular arithmetic functions
' ***************************************************************************************
DECLARE SUB Moddiv alias "Moddiv" (p AS SHORT, m AS SHORT)
'Compute positive residue p modulo largeint m. {t1..t2}
'
DECLARE SUB Modbal alias "Modbal" (p AS SHORT, m AS SHORT)
'Balanced residue p modulo m: |p| <= m / 2. {t1..t2}
'
DECLARE FUNCTION Mp2 alias "Mp2" (p AS SHORT, a AS SHORT) AS SHORT
'Return largeint p modulo a, which must be a one-word power of 2.
'
DECLARE SUB Modmlt alias "Modmlt" (p AS SHORT, q AS SHORT, m AS SHORT)
'Set largeint p to p * q (modulo m). {t1..t2}
'
DECLARE SUB Modsqu alias "Modsqu" (p AS SHORT, m AS SHORT)
'Set largeint p to p ^ 2 (modulo m). {t1..t2}
'
DECLARE SUB Modpwr alias "Modpwr" (p AS SHORT, q AS SHORT, m AS SHORT)
'Set largeint p to base p ^ exponent q (modulo m). {t0..t2, t3 if q < 0}
'
' ***************************************************************************************
'                             Number theoretic functions
' ***************************************************************************************
DECLARE SUB Powr alias "Powr" (p AS SHORT, k AS SHORT)
'Raise largeint p to the power |one-word k|. {t0..t3}
'
DECLARE SUB Fctl alias "Fctl" (p AS SHORT, a AS SHORT)
'Set largeint p to factorial a(a-1)21. {t1..t2}
'
DECLARE SUB Lcm alias "Lcm" (p AS SHORT, q AS SHORT, d AS SHORT)
'Set largeint d to the least common multiple (p, q). {t0..t2}
'
DECLARE SUB Gcd alias "Gcd" (p AS SHORT, q AS SHORT, d AS SHORT)
'Set largeint d to the greatest common divisor (p, q). {t0..t2}
'
DECLARE SUB Euclid alias "Euclid" (p AS SHORT, q AS SHORT, d AS SHORT)
'Apply the extended Euclidean algorithm to largeint's p and q,
'set p to 1/p (modulo q), q to q/gcd(p, q), and d to gcd(p, q). {t0..t2}
'
DECLARE SUB Bezout alias "Bezout" (p AS SHORT, q AS SHORT, d AS SHORT)
'Solution of the Diophantine equation px + qy = gcd(p, q),
'set largeint p to x, q to y, and d to gcd(p, q). {t0..t3}
'
DECLARE FUNCTION Kronec alias "Kronec" (p AS SHORT, q AS SHORT) AS SHORT
'Return Kronecker's quadratic residuosity symbol (p/q) = 0, 1, or -1. {t0..t3}
'
DECLARE FUNCTION IsSqr alias "IsSqr" (p AS SHORT, q AS SHORT) AS SHORT
'Boolean: return -1 if largeint p is a perfect square, else zero.
'If true, set q to the square root of p. {t0..t2}
'
DECLARE FUNCTION IsPPrm alias "IsPPrm" (p AS SHORT, d AS SHORT, k AS SHORT) AS SHORT
'Check primality of p with the Miller-Rabin test. Return 1 if
'p is definitely prime, -1 if probably prime, 0 if p is composite.
'k is the number of witnesses, k < 0 skips initial trial divisions. {t0..t3}
'
DECLARE FUNCTION Nxtprm alias "Nxtprm" (ByRef sw AS SHORT) AS INTEGER
'Loop through primetable PrimFlgs.bin. Initialize with sw = 0,
'then odd(sw) returns the next prime with each successive call.
'Use sw = 2 to push the current state and reset, sw =-2 to pop.
'
DECLARE FUNCTION PrimCeil alias "PrimCeil" () AS INTEGER
'Return the upper limit of primetable PrimFlgs.bin.
'
DECLARE SUB Triald alias "Triald" (q AS SHORT, p AS SHORT, c AS INTEGER)
'Trial divide p up to limit c. Primedivisors w/multiplicities are
'stored in list {q}, largeint p is set to the remaining cofactor. {t0..t3}
'
' ***************************************************************************************
'                                    Direct access
' ***************************************************************************************
DECLARE SUB EnQ alias "EnQ" (q AS SHORT, p AS SHORT, a AS SHORT)
'Store largeint p and |one-word a| in sequential list {q}.
'As {q} contains negative record separators, an attempt to
'simply 'Printn q, ...' will crash your program. Instead use:
'
DECLARE FUNCTION ExQ alias "ExQ" (p AS SHORT, ByRef a AS SHORT, q AS SHORT, _
 ByRef k AS SHORT) AS SHORT
'Get successive numbers p and |a| from list {q} at offset k.
'Set k = 0 to initialize reading, returns zero if @end-of-list.
'
DECLARE FUNCTION Getl alias "Getl" (p AS SHORT) AS SHORT
'Return the length of largeint p measured in words.
'
DECLARE FUNCTION Gets alias "Gets" (p AS SHORT) AS SHORT
'Return the sign bit of largeint p: -32768 if p < 0, else zero.
'
DECLARE FUNCTION Getw alias "Getw" (p AS SHORT, k AS SHORT) AS SHORT
'Return word number k of largeint p: a base MB digit (see below).
'
DECLARE SUB Setl alias "Setl" (p AS SHORT, a AS SHORT)
'Set the word length of largeint p to a.
'
DECLARE SUB Sets alias "Sets" (p AS SHORT, a AS SHORT)
'Set the sign bit of largeint p if a < 0, else clear.
'
DECLARE SUB Setw alias "Setw" (p AS SHORT, k AS SHORT, a AS SHORT)
'Set word number k of largeint p to |a| < MB.
'
' ***************************************************************************************
'                                  Internal functions
' ***************************************************************************************
DECLARE SUB Errorh alias "Errorh" (ByRef f AS STRING)
'Print an error message.
#IFDEF __FB_WIN32__
Declare Function ErrMsg lib "User32" alias "MessageBoxA" (hwnd As Integer, _
 lpText As Zstring Ptr, lpCaption As Zstring Ptr, wType As Integer) As Integer
#ENDIF
'
DECLARE SUB Lftj alias "Lftj" (p AS SHORT, k AS SHORT)
'Left-justify largeint p, beginning at word number k.
'
DECLARE FUNCTION Hp2 alias "Hp2" (p AS SHORT) AS SHORT
'Return 2^(highest set bit in the leftmost word of largeint p).
'
' ***************************************************************************************
#ELSE
'       : Same prototypes with -lang qb compatible argument types:
'
DEFINT A-B, D-Z: DEFLNG C
' ***************************************************************************************
DECLARE FUNCTION LargeInit alias "LargeInit" (ByVal k, ByVal f AS __ZSTRING __PTR) _
 AS INTEGER
DECLARE SUB Letf alias "Letf" (ByVal p, ByVal c)
DECLARE SUB Readst alias "Readst" (ByVal p, ByVal g AS __ZSTRING __PTR)
DECLARE SUB Rndf alias "Rndf" (ByVal p, ByVal k)
DECLARE SUB Term alias "Term" ()
' ***************************************************************************************
DECLARE FUNCTION Decf alias "Decf" (ByVal p) AS LONG
DECLARE SUB CnvSt alias "CnvSt" (ByVal g AS __ZSTRING __PTR)
DECLARE FUNCTION Logf alias "Logf" (ByVal p) AS DOUBLE
DECLARE SUB Ratdec alias "Ratdec" (ByVal g AS __ZSTRING __PTR, ByVal p, ByVal q)
' ***************************************************************************************
DECLARE SUB Printf alias "Printf" (f AS STRING, g AS STRING, h AS STRING, ByVal sw)
DECLARE SUB Printn alias "Printn" (ByVal p, f AS STRING, h AS STRING, ByVal sw)
DECLARE SUB Printr alias "Printr" (ByVal p, ByVal q, f AS STRING, ByVal k)
DECLARE SUB Prints alias "Prints" (f AS STRING, ByVal sw)
DECLARE SUB Work alias "Work" (f AS STRING)
' ***************************************************************************************
DECLARE SUB Subt alias "Subt" (ByVal p, ByVal q)
DECLARE SUB Add alias "Add" (ByVal p, ByVal q)
DECLARE SUB Inc alias "Inc" (ByVal p, ByVal a)
DECLARE SUB Divd alias "Divd" (ByVal p, ByVal d, ByVal q)
DECLARE FUNCTION Divint alias "Divint" (ByVal p, ByVal a) AS INTEGER
DECLARE SUB Ldiv alias "Ldiv" (ByVal p, ByVal d)
DECLARE SUB Mult alias "Mult" (ByVal p, ByVal q, ByVal r)
DECLARE SUB Lmul alias "Lmul" (ByVal p, ByVal q)
DECLARE SUB Squ alias "Squ" (ByVal p, ByVal q)
DECLARE SUB Isqrt alias "Isqrt" (ByVal p, ByVal q)
DECLARE SUB Chs alias "Chs" (ByVal p)
DECLARE FUNCTION Absf alias "Absf" (ByVal p) AS INTEGER
' ***************************************************************************************
DECLARE SUB Dup alias "Dup" (ByVal p, ByVal q)
DECLARE SUB Swp alias "Swp" (ByVal p, ByVal q)
DECLARE FUNCTION Cmp alias "Cmp" (ByVal p, ByVal q) AS INTEGER
DECLARE FUNCTION Isf alias "Isf" (ByVal p, ByVal a) AS INTEGER
DECLARE FUNCTION Ismin1 alias "Ismin1" (ByVal p, ByVal m) AS INTEGER
' ***************************************************************************************
DECLARE SUB Boolf alias "Boolf" (ByVal p, ByVal q, ByVal k)
DECLARE SUB Lsft alias "Lsft" (ByVal p, ByVal k)
DECLARE SUB Rsft alias "Rsft" (ByVal p, ByVal k)
DECLARE FUNCTION Odd alias "Odd" (ByVal p) AS INTEGER
DECLARE FUNCTION Bitl alias "Bitl" (ByVal p) AS LONG
' ***************************************************************************************
DECLARE SUB Moddiv alias "Moddiv" (ByVal p, ByVal m)
DECLARE SUB Modbal alias "Modbal" (ByVal p, ByVal m)
DECLARE FUNCTION Mp2 alias "Mp2" (ByVal p, ByVal a) AS INTEGER
DECLARE SUB Modmlt alias "Modmlt" (ByVal p, ByVal q, ByVal m)
DECLARE SUB Modsqu alias "Modsqu" (ByVal p, ByVal m)
DECLARE SUB Modpwr alias "Modpwr" (ByVal p, ByVal q, ByVal m)
' ***************************************************************************************
DECLARE SUB Powr alias "Powr" (ByVal p, ByVal k)
DECLARE SUB Fctl alias "Fctl" (ByVal p, ByVal a)
DECLARE SUB Lcm alias "Lcm" (ByVal p, ByVal q, ByVal d)
DECLARE SUB Gcd alias "Gcd" (ByVal p, ByVal q, ByVal d)
DECLARE SUB Euclid alias "Euclid" (ByVal p, ByVal q, ByVal d)
DECLARE SUB Bezout alias "Bezout" (ByVal p, ByVal q, ByVal d)
DECLARE FUNCTION Kronec alias "Kronec" (ByVal p, ByVal q) AS INTEGER
DECLARE FUNCTION IsSqr alias "IsSqr" (ByVal p, ByVal q) AS INTEGER
DECLARE FUNCTION IsPPrm alias "IsPPrm" (ByVal p, ByVal d, ByVal k) AS INTEGER
DECLARE FUNCTION Nxtprm alias "Nxtprm" (ByRef sw) AS LONG
DECLARE FUNCTION PrimCeil alias "PrimCeil" () AS LONG
DECLARE SUB Triald alias "Triald" (ByVal q, ByVal p, ByVal c)
' ***************************************************************************************
DECLARE SUB EnQ alias "EnQ" (ByVal q, ByVal p, ByVal a)
DECLARE FUNCTION ExQ alias "ExQ" (ByVal p, ByRef a, ByVal q, ByRef k) AS INTEGER
DECLARE FUNCTION Getl alias "Getl" (ByVal p) AS INTEGER
DECLARE FUNCTION Gets alias "Gets" (ByVal p) AS INTEGER
DECLARE FUNCTION Getw alias "Getw" (ByVal p, ByVal k) AS INTEGER
DECLARE SUB Setl alias "Setl" (ByVal p, ByVal a)
DECLARE SUB Sets alias "Sets" (ByVal p, ByVal a)
DECLARE SUB Setw alias "Setw" (ByVal p, ByVal k, ByVal a)
' ***************************************************************************************
DECLARE SUB Lftj alias "Lftj" (ByVal p, ByVal k)
DECLARE FUNCTION Hp2 alias "Hp2" (ByVal p) AS INTEGER
' ***************************************************************************************
#ENDIF
'
' ***************************************************************************************
'                                   Global constants
' ***************************************************************************************
CONST LB = 15, MB = &H8000& ' binary storage base
CONST LD = 4, MD = 10000 '    decimal string base < MB
CONST t0 = -1, t1 = -2 '      pointers to swap-registers
CONST t2 = -3, t3 = -4
'
' ***************************************************************************************
