' ******************************************************************
' Linear regression
' ******************************************************************

' ------------------------------------------------------------------
' Error codes
' ------------------------------------------------------------------

#DEFINE MatOk 0
' No error

#DEFINE MatSing -2
' Quasi-singular matrix

' ------------------------------------------------------------------
' Global variable
' ------------------------------------------------------------------

COMMON SHARED ErrCode AS INTEGER
' Error code from the last function evaluation

' ******************************************************************

SUB LinFit(X() AS DOUBLE, _
           Y() AS DOUBLE, _
           B() AS DOUBLE, _
           V() AS DOUBLE)
' ------------------------------------------------------------------
' Linear regression: Y = B(0) + B(1) * X
' ------------------------------------------------------------------
' Input parameters:  X(), Y() = point coordinates
' Output parameters: B()      = model parameters
'                    V()      = inverse matrix
' ------------------------------------------------------------------
' The function MathErr() returns one of the following codes:
'   MatOk   = no error
'   MatSing = singular matrix
' ------------------------------------------------------------------

  DIM AS DOUBLE  SX, SY, SX2, SXY, D
  DIM AS INTEGER K, N, Lb, Ub

  Lb = LBOUND(X)
  Ub = UBOUND(X)

  ErrCode = MatOk

  SX  = 0
  SY  = 0
  SX2 = 0
  SXY = 0

  N = Ub - Lb + 1

  FOR K = Lb TO Ub
    SX = SX + X(K)
    SY = SY + Y(K)
    SX2 = SX2 + X(K) * X(K)
    SXY = SXY + X(K) * Y(K)
  NEXT K

  D = N * SX2 - SX * SX

  IF D <= 0 THEN ErrCode = MatSing : EXIT SUB

  Lb = LBOUND(B)
  Ub = UBOUND(B)
  
  V(Lb, Lb) = SX2 / D
  V(Lb, Ub) = - SX / D
  V(Ub, Lb) = V(Lb, Ub)
  V(Ub, Ub) = N / D

  B(Lb) = V(Lb, Lb) * SY + V(Lb, Ub) * SXY
  B(Ub) = V(Ub, Lb) * SY + V(Ub, Ub) * SXY
END SUB
