' ******************************************************************
' Multiple linear regression
' ******************************************************************

' ------------------------------------------------------------------
' Error code
' ------------------------------------------------------------------

#DEFINE MatErrDim -3
' Non-compatible dimensions 

' ------------------------------------------------------------------
' Global variable
' ------------------------------------------------------------------

COMMON SHARED ErrCode AS INTEGER
' Error code from the last function evaluation

' ------------------------------------------------------------------
' External function
' ------------------------------------------------------------------

DECLARE SUB LinEq(      A() AS DOUBLE, _
                        B() AS DOUBLE, _
                  BYREF Det AS DOUBLE)
' Solves the linear system A * X = B

' ******************************************************************

SUB MulFit(X() AS DOUBLE, Y() AS DOUBLE, B() AS DOUBLE, V() AS DOUBLE)
' ------------------------------------------------------------------
' Multilinear regression
' ------------------------------------------------------------------
' Input : X() = Matrix of independent variables
'         Y() = Vector of dependent variable
' Output: B() = Vector of regression parameters
'         V() = Inverse matrix of normal equations
' ------------------------------------------------------------------

  DIM AS INTEGER Lb, Ub, LbY, UbY, I, J, K
  DIM AS DOUBLE  Det

  Lb = LBOUND(B)
  Ub = UBOUND(B)

  LbY = LBOUND(Y)
  UbY = UBOUND(Y)

  IF LBOUND(X, 1) <> LbY OR UBOUND(X, 1) <> UbY OR _
     LBOUND(X, 2) <> Lb  OR UBOUND(X, 2) <> Ub  OR _
     LBOUND(V, 1) <> Lb  OR UBOUND(V, 1) <> Ub  OR _
     LBOUND(V, 2) <> Lb  OR UBOUND(V, 2) <> Ub     _
  THEN
    ErrCode = MatErrDim
    EXIT SUB
  END IF

  ' Initialize
  FOR I = Lb TO Ub
    FOR J = I TO Ub
      V(I, J) = 0
    NEXT J
    B(I) = 0
  NEXT I
  
  ' Compute system of normal equations
  FOR K = LbY TO UbY
    FOR I = Lb TO Ub
      FOR J = I TO Ub
        V(I, J) = V(I, J) + X(K, I) * X(K, J)  ' X'WX
      NEXT J
      B(I) = B(I) + X(K, I) * Y(K)             ' X'WY
    NEXT I
  NEXT K

  ' Fill in symmetric matrix
  FOR I = Lb + 1 TO Ub
    FOR J = Lb TO I - 1
      V(I,J) = V(J,I)
    NEXT J
  NEXT I

  ' Solve system
  LinEq V(), B(), Det
END SUB
