' ******************************************************************
' Polynomial regression
' ******************************************************************

' ------------------------------------------------------------------
' Error code
' ------------------------------------------------------------------

#DEFINE MatErrDim -3
' Non-compatible dimensions

' ------------------------------------------------------------------
' Global variable
' ------------------------------------------------------------------

COMMON SHARED ErrCode AS INTEGER
' Error code from the last function evaluation

' ------------------------------------------------------------------
' External function
' ------------------------------------------------------------------

DECLARE SUB LinEq(      A() AS DOUBLE, _
                        B() AS DOUBLE, _
                  BYREF Det AS DOUBLE)
' Solves the linear system A * X = B

' ******************************************************************

SUB PolFit(X() AS DOUBLE,  _
           Y() AS DOUBLE,  _
           B() AS DOUBLE,  _
           V() AS DOUBLE)
' ------------------------------------------------------------------
' Polynomial regression: Y = B(0) + B(1) * X + B(2) * X^2 + ...
' ------------------------------------------------------------------
' Input parameters:  X(), Y() = point coordinates
' Output parameters: B()      = model parameters
'                    V()      = inverse matrix
' ------------------------------------------------------------------
' The function MathErr() returns one of the following codes:
'   MatOk     = no error
'   MatSing   = singular matrix
'   MatErrDim = non-compatible dimensions
' ------------------------------------------------------------------

  DIM AS INTEGER I, I1, J, K, Lb, Lb1, LbX, Ub, Ub1, UbX
  DIM AS DOUBLE  XI, Det

  Lb = LBOUND(B)
  Ub = UBOUND(B)

  LbX = LBOUND(X)
  UbX = UBOUND(X)

  IF LBOUND(Y)    <> LbX OR UBOUND(Y)    <> UbX OR _
     LBOUND(V, 1) <> Lb  OR UBOUND(V, 1) <> Ub  OR _
     LBOUND(V, 2) <> Lb  OR UBOUND(V, 2) <> Ub     _
  THEN
    ErrCode = MatErrDim
    EXIT SUB
  END IF

  ' Initialize
  FOR I = Lb TO Ub
    FOR J = Lb TO Ub
      V(I, J) = 0
    NEXT J
    B(I) = 0
  NEXT I
  
  Lb1 = Lb + 1    
  V(Lb, Lb) = UbX - LbX + 1       ' Number of observations
  FOR K = LbX TO UbX
    XI = X(K)                     ' x^i
    B(Lb) = B(Lb) + Y(K)
    V(Lb, Lb1) = V(Lb, Lb1) + XI
    B(Lb1) = B(Lb1) + XI * Y(K)
    FOR I = Lb + 2 TO Ub           
      XI = XI * X(K)                 
      V(Lb, I) = V(Lb, I) + XI    ' First line of matrix: 1 --> x^d
      B(I) = B(I) + XI * Y(K)     ' Constant vector: y --> x^d.y
    NEXT I                           
    FOR I = Lb1 TO Ub              
      XI = XI * X(K)              
      V(I, Ub) = V(I, Ub) + XI    ' Last column of matrix: x^d --> x^2d
    NEXT I
  NEXT K

 ' Fill lower matrix
  Ub1 = Ub - 1
  FOR I = Lb1 TO Ub    
    I1 = I - 1  
    FOR J = Lb TO Ub1
      V(I, J) = V(I1, J + 1)
    NEXT J
  NEXT I

  ' Solve system
  LinEq V(), B(), Det
END SUB
