' ******************************************************************
' Bisection method for nonlinear equation
' ******************************************************************

' ------------------------------------------------------------------
' Error codes
' ------------------------------------------------------------------

#DEFINE OptOk 0  
' No error

#DEFINE OptNonConv -1  
' Non-convergence

' ------------------------------------------------------------------
' Mathematical constant
' ------------------------------------------------------------------

#DEFINE Gold 1.618033988749895#   
' Golden Mean (1 + Sqrt(5))/2

' ------------------------------------------------------------------
' Global variable
' ------------------------------------------------------------------

COMMON SHARED ErrCode AS INTEGER
' Error code from the last function evaluation

' ******************************************************************

SUB RootBrack(      Func AS FUNCTION(X AS DOUBLE) AS DOUBLE,_
              BYREF X    AS DOUBLE,_
              BYREF Y    AS DOUBLE,_ 
              BYREF FX   AS DOUBLE,_
              BYREF FY   AS DOUBLE)
' ------------------------------------------------------------------
' Expands the interval [X,Y] until it contains a root of Func,
' i. e. Func(X) and Func(Y) have opposite signs. The corresponding
' function values are returned in FX and FY.
' ------------------------------------------------------------------
              
  FX = Func(X)
  FY = Func(Y)
  
  DO WHILE FX * FY > 0
    IF ABS(FX) < ABS(FY) THEN
      X = X + Gold * (X - Y)
      FX = Func(X)
    ELSE
      Y = Y + Gold * (Y - X)
      FY = Func(Y) 
    END IF
  LOOP
END SUB
              
SUB Bisect (      Func    AS FUNCTION(X AS DOUBLE) AS DOUBLE,_
            BYREF X       AS DOUBLE,_
            BYREF Y       AS DOUBLE,_
                  MaxIter AS INTEGER,_
                  Tol     AS DOUBLE,_
            BYREF F       AS DOUBLE)

  DIM AS INTEGER Iter
  DIM AS DOUBLE  G, Z, FZ

  Iter = 0
  ErrCode = OptOk

  F = Func(X)
  
  IF MaxIter < 1 THEN EXIT SUB
  
  G = Func(Y)
  
  IF F * G >= 0 THEN RootBrack(Func, X, Y, F, G)
  
  DO
    Iter = Iter + 1
    IF Iter > MaxIter THEN ErrCode = OptNonConv : EXIT SUB

    Z = 0.5 * (X + Y)
    FZ = Func(Z)
    
    IF F * FZ > 0 THEN 
      X = Z 
      F = FZ
    ELSE 
      Y = Z
      G = FZ
    END IF
  LOOP UNTIL ABS(X - Y) < Tol * (ABS(X) + ABS(Y))

  X = 0.5 * (X + Y)
  F = Func(X)
END SUB
