' ******************************************************************
' Secant method for nonlinear equation
' ******************************************************************

' ------------------------------------------------------------------
' Error codes
' ------------------------------------------------------------------

#DEFINE OptOk 0  
' No error

#DEFINE OptNonConv -1  
' Non-convergence

' ------------------------------------------------------------------
' Global variable
' ------------------------------------------------------------------

COMMON SHARED ErrCode AS INTEGER
' Error code from the last function evaluation

' ******************************************************************

SUB Secant (      Func    AS FUNCTION(X AS DOUBLE) AS DOUBLE,_
            BYREF X       AS DOUBLE,_
            BYREF Y       AS DOUBLE,_
                  MaxIter AS INTEGER,_
                  Tol     AS DOUBLE,_
            BYREF F       AS DOUBLE)

  DIM AS INTEGER Iter 
  DIM AS DOUBLE  G, Z

  Iter = 0
  ErrCode = OptOk

  DO
    F = Func(X)
   
    IF MaxIter < 1 THEN EXIT SUB
   
    G = Func(Y)
   
    IF F = G THEN ErrCode = OptNonConv : EXIT SUB
   
    Iter = Iter + 1
    IF Iter > MaxIter THEN ErrCode = OptNonConv : EXIT SUB

    Z = (X * G - Y * F) / (G - F)
   
    X = Y
    Y = Z
  LOOP UNTIL ABS(X - Y) < Tol * (ABS(X) + ABS(Y))

  X = 0.5 * (X + Y)
  F = Func(X)
END SUB
