' ******************************************************************
' Beta function 
' ******************************************************************

' ------------------------------------------------------------------
' Error code
' ------------------------------------------------------------------

#DEFINE FOk 0  
' No error

' ------------------------------------------------------------------
' Machine-dependent constant
' ------------------------------------------------------------------

#DEFINE MaxNum 1.797693134862315D+308   
' Max. floating point number: 2^1024

' ------------------------------------------------------------------
' Global variable
' ------------------------------------------------------------------

COMMON SHARED ErrCode AS INTEGER
' Error code from the latest function evaluation

' ------------------------------------------------------------------
' External functions
' ------------------------------------------------------------------

DECLARE FUNCTION SgnGamma(X AS DOUBLE) AS INTEGER
' Sign of Gamma function

DECLARE FUNCTION LnGamma(X AS DOUBLE) AS DOUBLE
' Ln of Gamma function

' ******************************************************************

FUNCTION Beta(X AS DOUBLE, Y AS DOUBLE) AS DOUBLE
' Computes Beta(X, Y) = Gamma(X) * Gamma(Y) / Gamma(X + Y)

DIM AS INTEGER SgnBeta
DIM AS DOUBLE  Lx, Ly, Lxy

Lxy = LnGamma(X + Y)
IF ErrCode <> FOk THEN
    Beta = 0
    EXIT FUNCTION
END IF

SgnBeta = SgnGamma(X) * SgnGamma(Y) * SgnGamma(X + Y)

Lx = LnGamma(X)
IF ErrCode <> FOk THEN
    Beta = SgnBeta * MaxNum
    EXIT FUNCTION
END IF

Ly = LnGamma(Y)
IF ErrCode <> FOk THEN
    Beta = SgnBeta * MaxNum
    EXIT FUNCTION
END IF

ErrCode = FOk
Beta = SgnBeta * EXP(Lx + Ly - Lxy)
END FUNCTION

