' ******************************************************************
' Factorial
' ******************************************************************

' ------------------------------------------------------------------
' Error codes
' ------------------------------------------------------------------

#DEFINE FOk 0
' No error

#DEFINE FDomain -1
' Argument domain error

#DEFINE FOverflow -3
' Overflow range error

' ------------------------------------------------------------------
' Machine-dependent constants
' ------------------------------------------------------------------

#DEFINE MaxNum 1.797693134862315D+308
' Max. floating point number: 2^1024

#DEFINE MaxFac 170
' Max. argument for Factorial

' ------------------------------------------------------------------
' Global variable
' ------------------------------------------------------------------

COMMON SHARED ErrCode AS INTEGER
' Error code from the latest function evaluation

' ------------------------------------------------------------------
' External function
' ------------------------------------------------------------------

DECLARE FUNCTION Gamma (X AS DOUBLE) AS DOUBLE
' Gamma function

' ******************************************************************

CONST NFact = 20

' Lookup table for factorials

DIM SHARED AS DOUBLE TabFact(0 TO NFact) = _
{1#, _
 1#, _
 2#, _
 6#, _
 24#, _
 120#, _
 720#, _
 5040#, _
 40320#, _
 362880#, _
 3628800#, _
 39916800#, _
 479001600#, _
 6227020800#, _
 87178291200#, _
 1307674368000#, _
 20922789888000#, _
 355687428096000#, _
 6.402373705728D+15, _
 1.21645100408832D+17, _
 2.43290200817664D+18}

FUNCTION Fact (N AS INTEGER) AS DOUBLE

  IF N < 0 THEN
    ErrCode = FDomain
    Fact = 1
    EXIT FUNCTION
  END IF

  IF N > MaxFac THEN
    ErrCode = FOverflow
    Fact = MaxNum
    EXIT FUNCTION
  END IF

  ErrCode = FOk

  IF N <= NFact THEN
    Fact = TabFact(N)
  ELSE
    Fact = Gamma(INT(N) + 1)
  END IF
END FUNCTION
