' ******************************************************************
' Logarithm of Gamma function
' Translated from C code in Cephes library (http://www.moshier.net)
' ******************************************************************

' ------------------------------------------------------------------
' Error codes
' ------------------------------------------------------------------

#DEFINE FOk 0  
' No error

#DEFINE FOverflow -3  
' Overflow range error

' ------------------------------------------------------------------
' Machine-dependent constant
' ------------------------------------------------------------------

#DEFINE MaxNum 1.797693134862315D+308   
' Max. floating point number: 2^1024

#DEFINE MaxLgm 2.556348D+305
' Max. argument for LnGamma

' ------------------------------------------------------------------
' Mathematical constants
' ------------------------------------------------------------------

#DEFINE Pi 3.141592653589793#   
' Pi

#DEFINE LnPi 1.1447298858494#     
' Ln(Pi)

#DEFINE LnSqrt2Pi 0.9189385332046728#  
' Ln(Sqrt(2*Pi))

' ------------------------------------------------------------------
' Global variable
' ------------------------------------------------------------------

COMMON SHARED ErrCode AS INTEGER
' Error code from the latest function evaluation

' ******************************************************************

FUNCTION StirLog (X AS DOUBLE) AS DOUBLE
' Approximates Ln(Gamma) with Stirling's formula, for X >= 13

DIM AS DOUBLE P, Q, W

Q = (X - 0.5) * LOG(X) - X + LnSqrt2Pi
IF X > 100000000 THEN
    StirLog = Q
ELSE
    W = 1 / (X * X)
    IF X > 1000 THEN
        P = 7.93650793650794D-04 * W - 2.77777777777778D-03
        P = P * W + 8.33333333333333D-02
    ELSE
        P = 8.11614167470509D-04 * W - 5.95061904284301D-04
        P = P * W + 7.93650340457717D-04
        P = P * W - .002777777777301#
        P = P * W + 8.33333333333332D-02
    END IF
    StirLog = Q + P / X
END IF
END FUNCTION

FUNCTION LnGamma (X AS DOUBLE) AS DOUBLE
' Logarithm of Gamma function

DIM AS DOUBLE A, N, P, Q, X1, Z

IF MaxLgm < X THEN GOTO Loverf
   
ErrCode = FOk

A = ABS(X)
IF A > 34 THEN
    IF X < 0 THEN
        N = INT(A)
        IF N = A THEN GOTO Loverf
        Z = A - N
        IF Z > .5 THEN N = N + 1: Z = N - A
        Z = A * SIN(Pi * Z)
        IF Z = 0 THEN GOTO Loverf
        Z = LnPi - LOG(Z) - StirLog(A)
    ELSE
        Z = StirLog(X)
    END IF
    LnGamma = Z
ELSEIF X < 13 THEN
    Z = 1
    X1 = X
    DO WHILE X1 >= 3
        X1 = X1 - 1
        Z = Z * X1
    LOOP
    DO WHILE X1 < 2
        IF X1 = 0 THEN GOTO Loverf
        Z = Z / X1
        X1 = X1 + 1
    LOOP
    IF Z < 0 THEN Z = -Z
    IF X1 = 2 THEN
        LnGamma = LOG(Z)
    ELSE
        X1 = X1 - 2
        P = -1378.25152569121# * X1 - 38801.6315134638#
        P = P * X1 - 331612.992738871#
        P = P * X1 - 1162370.97492762#
        P = P * X1 - 1721737.0082084#
        P = P * X1 - 853555.664245765#
        Q = X1 - 351.815701436524#
        Q = Q * X1 - 17064.2106651881#
        Q = Q * X1 - 220528.590553854#
        Q = Q * X1 - 1139334.44367983#
        Q = Q * X1 - 2532523.07177583#
        Q = Q * X1 - 2018891.41433533#
        LnGamma = LOG(Z) + X1 * P / Q
    END IF
ELSE
    LnGamma = StirLog(X)
END IF
EXIT FUNCTION

Loverf:
ErrCode = FOverflow
LnGamma = MaxNum
END FUNCTION
