' ******************************************************************
' Cholesky factorization of a positive definite symmetric matrix
' ******************************************************************

' ------------------------------------------------------------------
' Error codes
' ------------------------------------------------------------------

#DEFINE MatOk 0
' No error

#DEFINE MatNotPD -4
' Matrix not positive definite

' ------------------------------------------------------------------
' Global variable
' ------------------------------------------------------------------

COMMON SHARED ErrCode AS INTEGER
' Error code from the latest function evaluation

' ******************************************************************

SUB Cholesky(A() AS DOUBLE, L() AS DOUBLE)
' ------------------------------------------------------------------
' Cholesky decomposition. Factors the symmetric positive definite
' matrix A as a product L * L', where L is a lower triangular matrix
' This procedure may be used as a test of positive definiteness.
' ------------------------------------------------------------------
' Input parameter  : A = matrix
' Output parameter : L = Cholesky factor of matrix A
' ------------------------------------------------------------------
' The error code is returned in the global variable ErrCode:
'   MatOk     ==> no error
'   MatNotPD  ==> matrix A is not positive definite
' ------------------------------------------------------------------

  DIM AS INTEGER Lb, Ub   ' Array bounds
  DIM AS INTEGER I, J, K  ' Loop variables
  DIM AS DOUBLE  Sum

  Lb = LBOUND(A, 1)
  Ub = UBOUND(A, 1)

  FOR K = Lb TO Ub
    Sum = A(K, K)
    FOR J = Lb TO K - 1
      Sum = Sum - L(K, J) * L(K, J)
    NEXT J

    IF Sum <= 0 THEN ErrCode = MatNotPD : EXIT SUB

    L(K, K) = SQR(Sum)
    FOR I = K + 1 TO Ub
      Sum = A(I, K)
      FOR J = Lb TO K - 1
        Sum = Sum - L(I, J) * L(K, J)
      NEXT J
      L(I, K) = Sum / L(K, K)
    NEXT I
  NEXT K

  ErrCode = MatOk
END SUB

