' ******************************************************************
' Plotting routines : plot curve
' ******************************************************************

' ------------------------------------------------------------------
' Constant
' ------------------------------------------------------------------

#DEFINE LinScale 0
' Linear scale

' ------------------------------------------------------------------
' Global variables
' ------------------------------------------------------------------

COMMON SHARED AS INTEGER XScale, YScale
' Axis scales

' ------------------------------------------------------------------
' External functions
' ------------------------------------------------------------------

DECLARE FUNCTION MathErr() AS INTEGER
' Returns the error code from the last procedure call

DECLARE FUNCTION Log10(X AS DOUBLE) AS DOUBLE
' Decimal log

DECLARE FUNCTION Xpixel(X AS DOUBLE) AS INTEGER
' Converts user abscissa X to screen coordinate

DECLARE FUNCTION Ypixel(X AS DOUBLE) AS INTEGER
' Converts user ordinate Y to screen coordinate

DECLARE SUB PlotPoint(X      AS INTEGER, _
                      Y      AS INTEGER, _
                      Symbol AS INTEGER = 1, _
                      Size   AS INTEGER = 2)
' Plots a point

' ******************************************************************

SUB PlotCurve(X()     AS DOUBLE, _
              Y()     AS DOUBLE, _
              Symbol  AS INTEGER = 1, _
              Size    AS INTEGER = 2, _
              Connect AS INTEGER = 0)
' ------------------------------------------------------------------
' Plots a set of points
' ------------------------------------------------------------------
' Input parameters :
'
' X(), Y() : point coordinates in user units
'
' Connect  : True if the points must be connected by a line
'
' Other parameters as in PlotPoint
' ------------------------------------------------------------------

  DIM AS INTEGER I, Ierr, Lb, Ub, Xp, Yp, Xp1, Yp1
  DIM AS DOUBLE  Xi, Yi

  Ierr = 0
  
  Lb = LBOUND(X)
  Ub = UBOUND(X)

  FOR I = Lb TO Ub
    IF XScale = LinScale THEN
      Xi = X(I)
    ELSE
      Xi = Log10(X(I))
      Ierr = MathErr
    END IF

    IF YScale = LinScale THEN
      Yi = Y(I)
    ELSEIF Ierr = 0 THEN
      Yi = Log10(Y(I))
      Ierr = MathErr
    END IF

    IF Ierr = 0 THEN
      Xp = Xpixel(Xi)
      Yp = Ypixel(Yi)
      PlotPoint Xp, Yp, Symbol, Size
      IF Connect THEN
        IF I = Lb THEN PSET(Xp, Yp) ELSE LINE (Xp1, Yp1)-(Xp, Yp)
        Xp1 = Xp
        Yp1 = Yp
      END IF
    END IF
  NEXT I
END SUB

