' ******************************************************************
' Polynomials and rational fractions
' ******************************************************************

' ------------------------------------------------------------------
' Error codes
' ------------------------------------------------------------------

#DEFINE FOk 0  
' No error

#DEFINE FSing -2  
' Singularity

' ------------------------------------------------------------------
' Machine-dependent constant
' ------------------------------------------------------------------

#DEFINE MaxNum 1.797693134862315D+308   
' Max. floating point number: 2^1024

' ------------------------------------------------------------------
' Global variable
' ------------------------------------------------------------------

COMMON SHARED ErrCode AS INTEGER
' Error code from the latest function evaluation

' ******************************************************************

FUNCTION Poly(X      AS DOUBLE,_ 
              Coef() AS DOUBLE,_ 
              Deg    AS INTEGER) AS DOUBLE
' ------------------------------------------------------------------
' Evaluates the polynomial :
' P(X) = Coef(0) + Coef(1) * X + Coef(2) * X^2 +...
'          + Coef(Deg) * X^Deg
' ------------------------------------------------------------------

  DIM AS INTEGER I
  DIM AS DOUBLE  P

  P = Coef(Deg)
  FOR I = Deg - 1 TO 0 STEP -1
    P = P * X + Coef(I)
  NEXT I
  Poly = P
END FUNCTION

FUNCTION RFrac(X      AS DOUBLE,_
               Coef() AS DOUBLE,_ 
               Deg1   AS INTEGER,_ 
               Deg2   AS INTEGER) AS DOUBLE
' ------------------------------------------------------------------
' Evaluates the rational fraction :
'
'          Coef(0) + Coef(1) * X + ... + Coef(Deg1) * X^Deg1
' F(X) = -----------------------------------------------------
'        1 + Coef(Deg1+1) * X + ... + Coef(Deg1+Deg2) * X^Deg2
' ------------------------------------------------------------------

  DIM AS INTEGER I
  DIM AS DOUBLE  P, Q

  P = Coef(Deg1)
  FOR I = Deg1 - 1 TO 0 STEP -1
    P = P * X + Coef(I)
   NEXT I

  Q = 0
  FOR I = Deg1 + Deg2 TO Deg1 + 1 STEP -1
    Q = (Q + Coef(I)) * X
  NEXT I
  
  IF Q <> -1 THEN
    ErrCode = FOk
    RFrac = P / (1 + Q)
  ELSE
    ErrCode = FSing
    RFrac = SGN(P) * MaxNum
  END IF  
END FUNCTION

