' ******************************************************************
' Quadratic equation
' ******************************************************************

' ------------------------------------------------------------------
' Type definition
' ------------------------------------------------------------------

TYPE Complex
  X AS DOUBLE
  Y AS DOUBLE
END TYPE

' ------------------------------------------------------------------
' External function
' ------------------------------------------------------------------

DECLARE FUNCTION RootPol1(      A AS DOUBLE,_ 
                                B AS DOUBLE,_ 
                          BYREF X AS DOUBLE) AS INTEGER
' Linear equation

' ******************************************************************

FUNCTION RootPol2(Coef() AS DOUBLE, Z() AS Complex) AS INTEGER 
' ------------------------------------------------------------------
' Solves the quadratic equation:
' Coef(0) + Coef(1) * Z + Coef(2) * Z^2 = 0
' ------------------------------------------------------------------

  DIM AS DOUBLE Delta, F, Q

  Z(1).X = 0: Z(1).Y = 0
  Z(2).X = 0: Z(2).Y = 0

  IF Coef(2) = 0 THEN
    RootPol2 = RootPol1(Coef(0), Coef(1), Z(1).X)
    EXIT FUNCTION
  END IF

  IF Coef(0) = 0 THEN
    ' 0 is root. Eq. becomes linear
    IF RootPol1(Coef(1), Coef(2), Z(1).X) = 1 THEN
      ' Linear eq. has 1 solution
      RootPol2 = 2
    ELSE
      ' Linear eq. is undetermined or impossible
      RootPol2 = 1
    END IF
    EXIT FUNCTION
  END IF

  Delta = Coef(1) * Coef(1) - 4 * Coef(0) * Coef(2)

  ' 2 real roots
  IF Delta > 0 THEN
    RootPol2 = 2

    ' Algorithm for minimizing roundoff errors
    ' See `Numerical Recipes'
    IF Coef(1) >= 0 THEN
      Q = -0.5 * (Coef(1) + SQR(Delta))
    ELSE
      Q = -0.5 * (Coef(1) - SQR(Delta))
    END IF

    Z(1).X = Q / Coef(2)
    Z(2).X = Coef(0) / Q

    EXIT FUNCTION
  END IF

  ' Double real root
  IF Delta = 0 THEN
    RootPol2 = 2
    Z(1).X = -0.5 * Coef(1) / Coef(2)
    Z(2).X = Z(1).X
    EXIT FUNCTION
  END IF

  ' 2 complex roots
  RootPol2 = 0
  F = 0.5 / Coef(2)
  Z(1).X = -F * Coef(1)
  Z(1).Y = ABS(F) * SQR(-Delta)
  Z(2).X = Z(1).X
  Z(2).Y = -Z(1).Y
END FUNCTION
