' ******************************************************************
' Cubic equation
' ******************************************************************

' ------------------------------------------------------------------
' Type definition
' ------------------------------------------------------------------

TYPE Complex
  X AS DOUBLE
  Y AS DOUBLE
END TYPE

' ------------------------------------------------------------------
' External function
' ------------------------------------------------------------------

DECLARE FUNCTION RootPol2(Coef() AS DOUBLE,_ 
                          Z()    AS Complex) AS INTEGER  
' Quadratic equation

' ******************************************************************

FUNCTION RootPol3(Coef() AS DOUBLE, Z() AS Complex) AS INTEGER 
' ------------------------------------------------------------------
' Solves the cubic equation:
' Coef(0) + Coef(1) * Z + Coef(2) * Z^2 + Coef(3) * Z^3 = 0
' ------------------------------------------------------------------

  CONST OneThird  = 0.333333333333333#  ' 1 / 3
  CONST TwoPiDiv3 = 2.0943951023932#    ' 2 Pi / 3
  CONST Sqrt3Div2 = 0.866025403784439#  ' Sqrt(3) / 2

  DIM AS DOUBLE  A, AA, B, C, Q, QQQ, R, RR, S, T, U, Cf(0 TO 2)
  DIM AS INTEGER I

  FOR I = 1 TO 3
    Z(I).X = 0
    Z(I).Y = 0
  NEXT I

  IF Coef(3) = 0 THEN
    RootPol3 = RootPol2(Coef(), Z())
    EXIT FUNCTION
  END IF

  IF Coef(0) = 0 THEN
    ' 0 is root. Equation becomes quadratic
    Cf(0) = Coef(1): Cf(1) = Coef(2): Cf(2) = Coef(3)

    ' Solve quadratic equation
    RootPol3 = RootPol2(Cf(), Z()) + 1

    EXIT FUNCTION
  END IF

  IF Coef(3) = 1 THEN
    A = Coef(2) * OneThird
    B = Coef(1)
    C = Coef(0)
  ELSE
    A = Coef(2) / Coef(3) * OneThird
    B = Coef(1) / Coef(3)
    C = Coef(0) / Coef(3)
  END IF

  AA = A * A

  Q = AA - OneThird * B
  R = A * (AA - 0.5 * B) + 0.5 * C
  RR = R * R
  QQQ = Q * Q * Q

  IF RR < QQQ THEN                          ' 3 real roots
    RootPol3 = 3
    S = SQR(Q)
    T = R / (Q * S)
    T = ACOS(T)
    T = OneThird * T
    S = -2 * S
    Z(1).X = S * COS(T) - A
    Z(2).X = S * COS(T + TwoPiDiv3) - A
    Z(3).X = S * COS(T - TwoPiDiv3) - A
  ELSE                                      ' 1 real root
    RootPol3 = 1
    S = (ABS(R) + SQR(RR - QQQ)) ^ OneThird
    IF R > 0 THEN S = -S
    IF S = 0 THEN T = 0 ELSE T = Q / S
    U = S + T
    Z(1).X = U - A                           ' Real root
    Z(2).X = -0.5 * U - A
    Z(2).Y = Sqrt3Div2 * ABS(S - T)
    Z(3).X = Z(2).X
    Z(3).Y = -Z(2).Y
  END IF
END FUNCTION
