' ******************************************************************
' Binomial distribution
' ******************************************************************

' ------------------------------------------------------------------
' Error codes
' ------------------------------------------------------------------

#DEFINE FOk 0
' No error

#DEFINE FDomain -1
' Argument domain error

' ------------------------------------------------------------------
' Global variable
' ------------------------------------------------------------------

COMMON SHARED ErrCode AS INTEGER
' Error code from the latest function evaluation

' ******************************************************************

FUNCTION Binomial(N AS INTEGER, K AS INTEGER) AS DOUBLE
' ------------------------------------------------------------------
' Binomial coefficient C(N,K)
' ------------------------------------------------------------------

  DIM AS INTEGER I, K1, N1
  DIM AS DOUBLE  Prod 

  IF K < 0 THEN
    Binomial = 0
    ErrCode = FDomain
  ELSEIF K = 0 OR K = N THEN
    Binomial = 1
    ErrCode = FOk
  ELSEIF K = 1 OR K = N - 1 THEN
    Binomial = N
    ErrCode = FOk
  ELSE
    IF K > N - K THEN K1 = N - K ELSE K1 = K
    N1 = N + 1
    Prod = N
    FOR I = 2 TO K1
      Prod = (N1 - I) / I * Prod
    NEXT I
    Binomial = INT(0.5 + Prod)
    ErrCode = FOk
  END IF
END FUNCTION

FUNCTION PBinom(N AS INTEGER, P AS DOUBLE, K AS INTEGER) AS DOUBLE
' ------------------------------------------------------------------
' Probability of binomial distribution
' ------------------------------------------------------------------

  IF P < 0 OR P > 1 OR N <= 0 OR N < K THEN
    PBinom = 0
    ErrCode = FDomain
  ELSEIF K = 0 THEN
    PBinom = (1 - P) ^ N
    ErrCode = FOk
  ELSEIF K = N THEN
    PBinom = P ^ N
    ErrCode = FOk
  ELSE
    PBinom = Binomial(N, K) * (P ^ K) * (1 - P) ^ (N - K)
  END IF
END FUNCTION
