' ******************************************************************
'               Inverse of Khi-2 distribution function
' Translated from C code in Cephes library (http://www.moshier.net)
' ******************************************************************

' ------------------------------------------------------------------
' Error codes
' ------------------------------------------------------------------

#DEFINE FOk 0  
' No error

#DEFINE FDomain -1  
' Argument domain error

' ------------------------------------------------------------------
' Global variable
' ------------------------------------------------------------------

COMMON SHARED ErrCode AS INTEGER
' Error code from the latest function evaluation

' ------------------------------------------------------------------
' External function
' ------------------------------------------------------------------

DECLARE FUNCTION InvGamma(A AS DOUBLE, Y AS DOUBLE) AS DOUBLE

' ******************************************************************

FUNCTION InvKhi2(Nu AS INTEGER, P AS DOUBLE) AS DOUBLE
' ------------------------------------------------------------------
' Inverse of Khi-2 distribution function
'
' Returns the argument, X, for which the area under the Khi-2
' probability density function (integrated from 0 to X)
' is equal to P.
' ------------------------------------------------------------------

  IF P < 0 OR P > 1 OR Nu < 1 THEN
    ErrCode = FDomain
    InvKhi2 = 0
  ELSE
    ErrCode = FOk
    InvKhi2 = 2 * InvGamma(0.5 * Nu, P)
  END IF
END FUNCTION
