' ******************************************************************
'              Inverse of Snedecor distribution function
' Translated from C code in Cephes library (http://www.moshier.net)
' ******************************************************************

' ------------------------------------------------------------------
' Error codes
' ------------------------------------------------------------------

#DEFINE FOk 0  
' No error

#DEFINE FDomain -1  
' Argument domain error

' ------------------------------------------------------------------
' Global variable
' ------------------------------------------------------------------

COMMON SHARED ErrCode AS INTEGER
' Error code from the latest function evaluation

' ------------------------------------------------------------------
' External function
' ------------------------------------------------------------------

DECLARE FUNCTION InvBeta(A AS DOUBLE, B AS DOUBLE, Y AS DOUBLE) AS DOUBLE
' Inverse of incomplete Beta function

' ******************************************************************

FUNCTION InvSnedecor(Nu1 AS INTEGER, Nu2 AS INTEGER,_
                     P AS DOUBLE) AS DOUBLE
' ------------------------------------------------------------------
' Given probability P, finds the argument X such that
' FSnedecor(Nu, X) = P
' ------------------------------------------------------------------
  
  IF Nu1 < 1 OR Nu2 < 1 OR P <= 0 OR P > 1 THEN
    ErrCode = FDomain
    InvSnedecor = 0
    EXIT FUNCTION
  END IF

  DIM AS DOUBLE w

  ErrCode = FOk

  w = InvBeta(0.5 * Nu2, 0.5 * Nu1, 1 - P)
  InvSnedecor = (Nu2 - Nu2 * w) / (Nu1 * w)
END FUNCTION

