' ******************************************************************
'               Inverse of Student distribution function
' Translated from C code in Cephes library (http://www.moshier.net)
' ******************************************************************

' ------------------------------------------------------------------
' Error codes
' ------------------------------------------------------------------

#DEFINE FOk 0  
' No error

#DEFINE FDomain -1  
' Argument domain error

' ------------------------------------------------------------------
' Machine-dependent constant
' ------------------------------------------------------------------

#DEFINE MaxNum 1.797693134862315D+308
' Max. floating point number: 2^1024

' ------------------------------------------------------------------
' Global variable
' ------------------------------------------------------------------

COMMON SHARED ErrCode AS INTEGER
' Error code from the latest function evaluation

' ------------------------------------------------------------------
' External function
' ------------------------------------------------------------------

DECLARE FUNCTION InvBeta(A AS DOUBLE, B AS DOUBLE, Y AS DOUBLE) AS DOUBLE

' ******************************************************************

FUNCTION InvStudent(Nu AS INTEGER, P AS DOUBLE) AS DOUBLE
' ------------------------------------------------------------------
' Given probability P, finds the argument X such that
' FStudent(Nu, X) = P
' ------------------------------------------------------------------

  IF Nu <= 0 OR P <= 0 OR P >= 1 THEN
    ErrCode = FDomain
    InvStudent = 0
    EXIT FUNCTION
  END IF

  DIM AS DOUBLE  t, rk, z
  DIM AS INTEGER rflg

  ErrCode = FOk
  rk = Nu

  IF P > 0.25 AND P < 0.75 THEN
    IF P = 0.5 THEN InvStudent = 0 : EXIT FUNCTION
    z = 1 - 2 * P
    z = InvBeta(0.5, 0.5 * rk, ABS(z))
    t = SQR(rk * z / (1 - z))
    IF P < 0.5 THEN t = -t
    InvStudent = t
    EXIT FUNCTION
  END IF

  IF P < 0.5 THEN
    z = P
    rflg = -1
  ELSE
    z = 1 - P
    rflg = 1
  END IF

  z = InvBeta(0.5 * rk, 0.5, 2 * z)

  IF MaxNum * z < rk THEN InvStudent = rflg * MaxNum : EXIT FUNCTION

  t = SQR(rk / z - rk)
  InvStudent = rflg * t
END FUNCTION

