' ******************************************************************
' Bartlett's test (comparison of several variances)
' ******************************************************************

' ------------------------------------------------------------------
' Error codes
' ------------------------------------------------------------------

#DEFINE FOk 0
' No error

#DEFINE FSing -2
' Singularity

#DEFINE MatErrDim -3
' Non-compatible dimensions

' ------------------------------------------------------------------
' Global variable
' ------------------------------------------------------------------

COMMON SHARED ErrCode AS INTEGER
' Error code from the latest function evaluation

' ******************************************************************

SUB Bartlett(      N()  AS INTEGER, _
                   S()  AS DOUBLE,  _
             BYREF Khi2 AS DOUBLE,  _
             BYREF Dof  AS INTEGER)
' ------------------------------------------------------------------
' Input parameters : N()  = samples sizes
'                    S()  = samples SD's (computed with StDev) 
' Output parameters: Khi2 = Bartlett's khi-2
'                    DoF  = degrees of freedom
' ------------------------------------------------------------------    

  DIM AS INTEGER Lb, Ub, I, Ns, Nt

  Lb = LBOUND(N)
  Ub = UBOUND(N)

  Ns = Ub - Lb + 1  ' Number of samples

  IF Ns < 2 OR LBOUND(S) <> Lb OR UBOUND(S) <> Ub THEN 
    ErrCode = MatErrDim
    EXIT SUB
  END IF

  Nt = 0
  FOR I = Lb TO Ub
    Nt = Nt + N(I)
  NEXT I

  IF Nt <= Ns THEN
    ErrCode = FSing
    EXIT SUB
  END IF

  DIM AS INTEGER N1, DoF_r
  DIM AS DOUBLE  SSr, Vr, Vi, SumLog, SumInv

  ErrCode = FOk

  SSr = 0
  SumLog = 0
  SumInv = 0

  FOR I = Lb TO Ub
    N1 = N(I) - 1
    Vi = S(I) * S(I)
    SSr = SSr + N1 * Vi
    SumLog = SumLog + N1 * LOG(Vi)
    SumInv = SumInv + 1 / N1
  NEXT I

  DoF = Ns - 1
  DoF_r = Nt - Ns
  Vr = SSr / DoF_r
  Khi2 = (DoF_r * LOG(Vr) - SumLog) / (1 + (SumInv - 1 / DoF_r) / (3 * DoF))
END SUB
