' ******************************************************************
' Student t-test for paired samples
' ******************************************************************

' ------------------------------------------------------------------
' Error codes
' ------------------------------------------------------------------

#DEFINE FOk 0
' No error

#DEFINE FSing -2
' Singularity

#DEFINE MatErrDim -3
' Non-compatible dimensions

' ------------------------------------------------------------------
' Machine-dependent constant
' ------------------------------------------------------------------

#DEFINE MaxNum 1.797693134862315D+308
' Max. floating point number: 2^1024

' ------------------------------------------------------------------
' Global variable
' ------------------------------------------------------------------

COMMON SHARED ErrCode AS INTEGER
' Error code from the latest function evaluation

' ------------------------------------------------------------------
' External functions
' ------------------------------------------------------------------

DECLARE FUNCTION Mean(X() AS DOUBLE) AS DOUBLE
' Mean of sample X

DECLARE FUNCTION StDev(X() AS DOUBLE, M AS DOUBLE) AS DOUBLE
' Standard deviation estimated from sample X

' ******************************************************************

SUB StudPaired(      X() AS DOUBLE, _
                     Y() AS DOUBLE, _
               BYREF T   AS DOUBLE, _
               BYREF DoF AS INTEGER)
' ------------------------------------------------------------------
' Input parameters : X(), Y() = samples to be compared
' Output parameters: T        = Student's t
'                    DoF      = degrees of freedom
' ------------------------------------------------------------------

  DIM AS INTEGER Lb, Ub, I, N

  Lb = LBOUND(X)
  Ub = UBOUND(X)

  DoF = Ub - Lb  ' N - 1
  N = DoF + 1

  IF DoF = 0 OR LBOUND(Y) <> Lb OR UBOUND(Y) <> Ub THEN
    ErrCode = MatErrDim
    EXIT SUB
  END IF

  DIM AS DOUBLE D(Lb TO Ub)  ' Differences X - Y
  DIM AS DOUBLE MD, SD       ' Mean and st. dev. of D

  FOR I = Lb TO Ub
    D(I) = X(I) - Y(I)
  NEXT I

  MD = Mean(D())
  SD = StDev(D(), MD)

  IF SD > 0 THEN
    T = MD * SQR(N) / SD
    ErrCode = FOk
  ELSE
    T = SGN(MD) * MaxNum
    ErrCode = FSing
  END IF
END SUB

